@@ -10,7 +10,7 @@ class CustomRenderTest < Redcarpet::TestCase
 
   def test_simple_overload
     md = Redcarpet::Markdown.new(SimpleRender)
-    html_equal "<p>This is <em class=\"cool\">just</em> a test</p>\n",
+    assert_equal "<p>This is <em class=\"cool\">just</em> a test</p>\n",
       md.render("This is *just* a test")
   end
 
